/*
 * inverter.c
 *
 *  Created on: 19.09.2016
 *      Author: dehottgw
 */


#include <stdint.h>
#include <stdlib.h>
#include <msp430g2452.h>
#include <isr_compat.h>

#include "inverter.h"

#define NUM_OF_SINE_VALUES 20

float sineValues[NUM_OF_SINE_VALUES] = {
		0.1564,
		0.3090,
		0.4540,
		0.5878,
		0.7071,
		0.8090,
		0.8910,
		0.9511,
		0.9877,
		1.0000,
		0.9877,
		0.9511,
		0.8910,
		0.8090,
		0.7071,
		0.5878,
		0.4540,
		0.3090,
		0.1564,
		0.0000
};

volatile uint16_t pulseWidths[NUM_OF_SINE_VALUES * 2];
volatile uint8_t pulseWidthIdx = 0;
volatile uint8_t activeOffset = 0;



ISR(TIMER0_A0, TA0_ISR_Ovrfl) {
	if (pulseWidthIdx == 0) {
	}

	uint16_t pw = pulseWidths[pulseWidthIdx + activeOffset];
	if (pw != 0) {
		TA0CCR1 = pw;
		P2OUT |= BIT1;
	}

	pulseWidthIdx++;
	if (pulseWidthIdx >= NUM_OF_SINE_VALUES) {
		pulseWidthIdx = 0;
	}
}

ISR(TIMER0_A1, TA0_ISR_Comp) {
	__disable_interrupt();
	uint16_t taiv = TA0IV;
	if (taiv & TA0IV_TACCR1) {
		P2OUT &= ~BIT1;
	} else if ((taiv && TA0IV_TACCR2) && (pulseWidthIdx == 0)) {
		P1OUT ^= (BIT1 | BIT2);
	}
	__enable_interrupt();
}



void inverterInit() {
	P1OUT &= ~BIT2;
	P1OUT |= BIT1;
	P1DIR |= BIT1 | BIT2;

	P2OUT &= ~BIT1;
	P2DIR |= BIT1;

	// TA0OV1 out
	P1OUT &= ~BIT5;
	P1DIR |= BIT5;
	P1SEL |= BIT5;

	//TACLK in
	P1OUT &= ~BIT0;
	P1DIR &= ~BIT0;
	P1SEL |= BIT0;

	// SMCLK out
	P1DIR |= BIT4;
	P1SEL |= BIT4;

	TA0CTL = MC_1 | ID_0 | TASSEL_0 | TACLR;
	TA0CCTL0 = CCIE | OUTMOD_4;
	TA0CCTL1 = CCIE;
	TA0CCTL2 = CCIE;
	TA0CCR0 = 0;
	TA0CCR1 = 0;
	TA0CCR2 = 1;
}

void inverterSetFrequency(uint16_t f) {
	float ff = (float)f;
	float slotLength = 2 / (ff * 400e-9 * NUM_OF_SINE_VALUES);
	uint16_t sl = (uint16_t)slotLength;

	uint8_t currentOffset = (activeOffset == 0) ? NUM_OF_SINE_VALUES : 0;
	for (uint8_t i = 0; i < NUM_OF_SINE_VALUES; i++) {
		uint16_t pw = (uint16_t)(slotLength * 0.9 * sineValues[i]);
		pulseWidths[i + currentOffset] = pw;
	}

	__disable_interrupt();
	activeOffset = currentOffset;
	TA0CCR0 = sl;
	pulseWidthIdx = 0;
	__enable_interrupt();
}
